package jp.co.sds_corp.util.audio;

import java.io.File;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.SourceDataLine;

/**
* Audioファイルの再生を行います。
* @author	小見山　聡
*/
public class AudioPlay extends Thread
{
	SourceDataLine	line;
//	Clip			clip;
	FloatControl	ctr;
	float			fCur;
	float			fVolume;
	File	af;
	AudioFormat		fmt;
	AudioInputStream	ais;
	int		nTimes = -1;

	/**
	* ボリュームの値をdB(デシベル)に変換します。
	* @param	val		ボリュームの値
	* @return	dB値
	*/
	float getdB( float val ) {
		return (float)(Math.log(val==0.0?0.0001:val)/Math.log(10.0)*20.0);
	}

	/**
	* Audioファイルを指定された回数再生するよう初期化します。
	* @param	fname	再生するAudioファイルの名前。
	* @param	nTimes	再生回数。-1以下を指定すると無限ループします。
	* @param	volume	ボリューム値
	*/
	public AudioPlay( String fname, int nTimes, float volume )
		throws Exception
	{
		this( fname, nTimes );
		this.fVolume = volume;
	}

	/**
	* Audioファイルを指定された回数再生するよう初期化します。
	* @param	fname	再生するAudioファイルの名前。
	* @param	nTimes	再生回数。-1以下を指定すると無限ループします。
	*/
	public AudioPlay( String fname, int nTimes ) throws Exception {
		this( fname );
		this.nTimes = nTimes;
	}

	/**
	* Audioファイルを無限に再生するよう初期化します。
	* @param	fname	再生するAudioファイルの名前。
	*/
	public AudioPlay( String fname ) throws Exception {
		nTimes = -1;
		af = new File( fname );
	}

	protected AudioInputStream openSource() throws Exception {
		AudioInputStream ais;
		ais = AudioSystem.getAudioInputStream( af );
		fmt = ais.getFormat();
		if (( fmt.getEncoding() == AudioFormat.Encoding.ULAW ) ||
			( fmt.getEncoding() == AudioFormat.Encoding.ALAW ))
		{
			AudioFormat tmp = new AudioFormat(
									AudioFormat.Encoding.PCM_SIGNED,
									fmt.getSampleRate(),
									fmt.getSampleSizeInBits() * 2,
									fmt.getChannels(),
									fmt.getFrameSize() * 2,
									fmt.getFrameRate(),
									true );
			ais = AudioSystem.getAudioInputStream( tmp, ais );
			fmt = tmp;
		}
		return ais;
	}

	protected void openLine() throws Exception {
/*
		DataLine.Info info = new DataLine.Info( Clip.class, fmt );
*/
		DataLine.Info info = new DataLine.Info( SourceDataLine.class, fmt );

		if ( !AudioSystem.isLineSupported( info )) {
			// Handle the error.
			System.out.println( "error" );
			return;
		}

		// Obtain and open the line.
/*
		clip = (Clip )AudioSystem.getLine( info );
		clip.open( ais );
		ctr = (FloatControl )clip.getControl( FloatControl.Type.MASTER_GAIN );
*/
		line = (SourceDataLine )AudioSystem.getLine( info );
		line.open( fmt );
		ctr = (FloatControl )line.getControl( FloatControl.Type.MASTER_GAIN );
		fVolume = ctr.getMaximum();
		fCur = getdB( ctr.getValue());
	}

	boolean		bPlay = false;

	/**
	* 演奏中フラグを取得します。
	* @return	演奏中フラグ
	*/
	synchronized public boolean getPlayFlg() {
		return bPlay;
	}
	/**
	* 演奏中フラグを設定します。
	* @param	b		演奏中フラグ
	*/
	synchronized public void setPlayFlg( boolean b ) {
		bPlay = b;
	}

	/**
	* 演奏を実行します。
	*/
	public void run() {
		try {
			setPlayFlg( true );
			ais = openSource();
			openLine();
			ctr.setValue( fVolume );
			line.start();
			int bufSize = (int )fmt.getSampleRate() * fmt.getFrameSize();
			byte[] buf	= new byte[bufSize];
			sleep( 99 );
			while ( this == Thread.currentThread()) {
				if( !getPlayFlg() ) {
					break;
				}
				if ( nTimes == 0 ) {
					break;
				}
				nTimes--;
				int		nRead;
				ais.close();
				ais = openSource();
				ais.mark( bufSize );
				for ( ;; ) {
					nRead = ais.read( buf, 0, buf.length );
					if ( nRead < 0 ) {
						break;
					}
					line.write( buf, 0, nRead );
				}
				line.drain();
				sleep( 1000 );
				yield();
			}
			ais.close();
			line.close();
		} catch ( InterruptedException ex ) {
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

}

