package	jp.co.sds_corp.util.calc;


/**
* 計算式からCalcNodeの二分木を生成します。
*/
public class CalcCompiler
{
	String exp;

	public CalcCompiler() {
	}

	/**
	* 計算式からCalcNodeの二分木を生成します。
	* @param	exp		計算式
	*/
	public CalcNode compile( String exp ) throws Exception {
		this.exp = exp;
		return compile();
	}

	/**
	* 計算式からCalcNodeの二分木を生成します。
	*/
	public CalcNode compile() throws Exception {
		CalcNode topNode = createNode( exp );
		return topNode;
	}

	/**
	* 計算式からCalcNodeを生成します。
	* @param	exp		計算式
	*/
	protected CalcNode createNode( String exp )
		throws Exception
	{
		CalcNode topNode = null;
		CalcNode prevNode = null;
		CalcNode prevOpe = null;
		CalcNode node = null;
		Tokenizer tokenizer = new Tokenizer( exp );
		for ( ;; ) {
			String tok = tokenizer.nextToken();
			if ( tokenizer.getType() == Tokenizer.TYPE_EOF ) {
				break;
			}
			switch ( tokenizer.getType() ) {
			case Tokenizer.TYPE_NUMBER:
				node = new DecimalNode( tok );
				addNode( prevOpe, node );
				break;
			case Tokenizer.TYPE_LITERAL:
				String next = tokenizer.peekToken();
				if ( next != null && next.equals( "(" )) {
					tokenizer.nextToken();
					String blank = tokenizer.searchBlanket();
					CalcNode[] arr = null;
					if ( blank == null || blank.length() > 0 ) {
						arr = createNodeArray( blank );
					} else {
						arr = new CalcNode[0];
					}
					node = new FunctionNode( tok, arr );
					addNode( prevOpe, node );
				} else {
					node = new StringNode( tok );
					addNode( prevOpe, node );
				}
				break;
			default:
				if ( tok.equals( "(" )) {
					tok = tokenizer.searchBlanket();
					node = createNode( tok );
					node.level = 99;
					addNode( prevOpe, node );
				} else {
					node = new OperatorNode( tok );
					if ( prevOpe == null ) {
						node.child1 = prevNode;
						topNode = node;
					} else {
						if ( node.getLevel() > prevOpe.getLevel() ) {
							addOpe( prevOpe, node );
						} else {
							if ( prevOpe.parent == null ) {
								node.child1 = topNode;
								topNode = node;
							} else {
								addOpe( prevOpe.parent, node );
							}
						}
					}
					prevOpe = node;
				}
			}
			if ( prevOpe == null ) {
				topNode = node;
			}
			prevNode = node;
		}
		return topNode;
	}

	private void addNode( CalcNode prevOpe, CalcNode node ) {
		if ( prevOpe != null ) {
			prevOpe.child2 = node;
			node.parent = prevOpe;
		}
	}

	private void addOpe( CalcNode prevOpe, CalcNode node ) {
		if ( prevOpe != null ) {
			node.child1 = prevOpe.child2;
			prevOpe.child2 = node;
			node.parent = prevOpe;
		}
	}

	protected CalcNode[] createNodeArray( String exp )
		throws Exception
	{
		CalcNode[] ret;
		// @@@ とりあえず
		String[] arr = exp.split( "," );
		ret = new CalcNode[arr.length];
		for ( int i = 0; i < arr.length; i++ ) {
			ret[i] = createNode( arr[i] );
		}
		return ret;
	}

}
