package jp.co.sds_corp.util.charset;

import java.io.BufferedInputStream;
import java.io.FileInputStream;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class BOMChecker
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	public enum BOM {
		NONE	(new byte[0]),
		UTF_8	(new byte[]{(byte )0xEF, (byte )0xBB, (byte )0xBF}),
		UTF_16_BE(new byte[]{(byte )0xFE, (byte )0xFF}),
		UTF_16_LE(new byte[]{(byte )0xFF, (byte )0xFE}),
		UTF_32_BE(new byte[]{(byte )0x00, (byte )0x00, (byte )0xFE, (byte )0xFF}),
		UTF_32_LE(new byte[]{(byte )0xFF, (byte )0xFE, (byte )0x00, (byte )0x00}),
		UTF_7	(new byte[]{(byte )0x2B, (byte )0x2F, (byte )0x76}),
		;

		byte[] bytes;
		BOM(byte[] bytes) {
			this.bytes = bytes;
		}
		public byte[] getBytes() {
			return bytes;
		}
	}

	public void check(String filename, BOM bom) throws CheckFailureException {
		BOM parsedBom = getBOM(filename);
		if (parsedBom != bom) {
			throw(new CheckFailureException("not expected BOM. expected:" + bom + " actual:" + parsedBom));
		}
	}

	public BOM getBOM(String filename) throws CheckFailureException {
		BOM ret = BOM.NONE;
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(filename);
			BufferedInputStream bis = new BufferedInputStream(fis);
			byte[] inBuf = new byte[16];
			int len = bis.read(inBuf);
			if (logger.isTraceEnable(this)) {
				StringBuilder buf = new StringBuilder();
				for (int i = 0; i < len; i++) {
					buf.append(Integer.toHexString(inBuf[i] & 0xff)).append(" ");
				}
				logger.trace(this, "buf=" + buf);
			}
			for (BOM bom : BOM.values()) {
				byte[] bomBytes = bom.getBytes();
				if (bomBytes.length > 0) {
					boolean unmatch = false;
					for (int i = 0; i < bomBytes.length; i++) {
						if (i >= len || inBuf[i] != bomBytes[i]) {
							unmatch = true;
							break;
						}
					}
					if (!unmatch) {
						ret = bom;
						break;
					}
				}
			}
			bis.close();
		} catch (Exception ex) {
			throw(new CheckFailureException("file read error. " + filename, ex));
		} finally {
			if (fis == null) {
				try {
					fis.close();
				} catch (Exception ex) {
					logger.error(this, "close error. " + filename, ex);
				}
			}
		}
		return ret;
	}

}
