package jp.co.sds_corp.util.charset;

import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CodingErrorAction;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class CharsetWrapper extends Charset {
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Charset charset;
	CharsetEncoder encoder;
	CharsetDecoder decoder;

	public CharsetWrapper(String canonicalName, String[] alias) {
		super(canonicalName, alias);
		this.charset = Charset.forName("ISO-8859-1");
	}

	public CharsetWrapper(Charset charset) {
		super(charset.name(), charset.aliases().toArray(new String[0]));
		this.charset = charset;
	}

	@Override
	public boolean contains(Charset cs) {
		return charset.contains(cs);
	}

	@Override
	public CharsetDecoder newDecoder() {
		logger.debug(this, "newDecoder " + name());
		decoder = charset.newDecoder()
				.onMalformedInput(CodingErrorAction.REPORT)
				.onUnmappableCharacter(CodingErrorAction.REPORT);
		decoder = new CharsetDecoderWrapper(decoder);
		return decoder;
	}

	@Override
	public CharsetEncoder newEncoder() {
		logger.debug(this, "newEncoder " + name());
		encoder = charset.newEncoder()
				.onMalformedInput(CodingErrorAction.REPORT)
				.onUnmappableCharacter(CodingErrorAction.REPORT);
		encoder = new CharsetEncoderWrapper(encoder);
		return encoder;
	}
}
