package jp.co.sds_corp.util.config;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;

/**
 * Documentクラス
 */
public class ConfigurationDocument extends ConfigurationNode implements Document
{
	Document doc;
	Map<String,String> mapProp = new HashMap<String,String>();
	static Pattern pattern = Pattern.compile( "\\$\\{([\\w.]+)\\}" );

	protected ConfigurationDocument( Document doc ) throws DOMException {
		super( doc, null );
		this.doc = doc;
	}

	protected void setProperty( String key, String value ) {
		mapProp.put( key, value );
	}

	protected String getProperty( String key ) {
		String ret = mapProp.get( key );
		return ret;
	}

	/**
	* ${foo}形式の値を、property要素で指定された内容で置き換えます。
	* @param    value   置き換える文字列
	* @return   置き換え後の文字列
	*/
	protected String replaceValue( String value ) {
		Matcher matcher = pattern.matcher( value );
		if ( !matcher.find()) {
			return value;
		}
		String key = matcher.group( 1 );
		String var = getProperty( key );
		if ( var == null ) {
			var = System.getProperty( key );
			if ( var == null ) {
				return value;
			}
		}
		String ret = matcher.replaceFirst( var );
		return replaceValue( ret );
	}

	@Override
	public Node adoptNode(Node source) {
		return doc.adoptNode( source );
	}
	@Override
	public Attr createAttribute(String name) {
		return doc.createAttribute( name );
	}
	@Override
	public Attr createAttributeNS(String namespaceURI, String qualifiedName) {
		return doc.createAttributeNS( namespaceURI, qualifiedName );
	}
	@Override
	public CDATASection createCDATASection(String data) {
		return doc.createCDATASection( data );
	}
	@Override
	public Comment createComment(String data) {
		return doc.createComment( data );
	}
	@Override
	public DocumentFragment createDocumentFragment() {
		return doc.createDocumentFragment();
	}
	@Override
	public Element createElement(String tagName) {
		return doc.createElement( tagName );
	}
	@Override
	public Element createElementNS(String namespaceURI, String qualifiedName) {
		return doc.createElementNS( namespaceURI, qualifiedName );
	}
	@Override
	public EntityReference createEntityReference(String name) {
		return doc.createEntityReference( name );
	}
	@Override
	public ProcessingInstruction createProcessingInstruction(String target, String data) {
		return doc.createProcessingInstruction( target, data );
	}
	@Override
	public Text createTextNode(String data) {
		return doc.createTextNode( data );
	}
	@Override
	public DocumentType getDoctype() {
		return doc.getDoctype();
	}
	@Override
	public Element getDocumentElement() {
		return doc.getDocumentElement();
	}
	@Override
	public String getDocumentURI() {
		return doc.getDocumentURI();
	}
	@Override
	public DOMConfiguration getDomConfig() {
		return doc.getDomConfig();
	}
	@Override
	public Element getElementById(String elementId) {
		return doc.getElementById( elementId );
	}
	@Override
	public NodeList getElementsByTagName(String tagname) {
		return doc.getElementsByTagName( tagname );
	}
	@Override
	public NodeList getElementsByTagNameNS(String namespaceURI, String localName) {
		return doc.getElementsByTagNameNS( namespaceURI, localName );
	}
	@Override
	public DOMImplementation getImplementation() {
		return doc.getImplementation();
	}
	@Override
	public String getInputEncoding() {
		return doc.getInputEncoding();
	}
	@Override
	public boolean getStrictErrorChecking() {
		return doc.getStrictErrorChecking();
	}
	@Override
	public String getXmlEncoding() {
		return doc.getXmlEncoding();
	}
	@Override
	public boolean getXmlStandalone() {
		return doc.getXmlStandalone();
	}
	@Override
	public String getXmlVersion() {
		return doc.getXmlVersion();
	}
	@Override
	public Node importNode(Node importedNode, boolean deep) {
		return doc.importNode( importedNode, deep );
	}
	@Override
	public void normalizeDocument() {
		doc.normalizeDocument();
	}
	@Override
	public Node renameNode(Node n, String namespaceURI, String qualifiedName) {
		return doc.renameNode( n, namespaceURI, qualifiedName );
	}
	@Override
	public void setDocumentURI(String documentURI) {
		doc.setDocumentURI( documentURI );
	}
	@Override
	public void setStrictErrorChecking(boolean strictErrorChecking) {
		doc.setStrictErrorChecking( strictErrorChecking );
	}
	@Override
	public void setXmlStandalone(boolean xmlStandalone) {
		doc.setXmlStandalone( xmlStandalone );
	}
	@Override
	public void setXmlVersion(String xmlVersion) {
		doc.setXmlVersion( xmlVersion );
	}
}
