package jp.co.sds_corp.util.db;

import java.sql.SQLException;

/**
* ＤＢアクセスで例外的条件が発生した場合にスローされます。
*/
public class DbException extends Exception
{
	private static final long serialVersionUID = 1L;
	private String		sql = null;

	/**
	* 詳細メッセージを指定しないで DbException を構築します。
	*/
	public DbException() {
		super();
	}

	/**
	* 指定された詳細メッセージを使用して、DbExceptionを構築します。
	* @param	msg		詳細メッセージ
	*/
	public DbException( String msg ) {
		super( msg );
	}

	/**
	* 指定された詳細メッセージと原因を使用して、DbExceptionを構築します。
	* @param	msg		詳細メッセージ
	* @param	ex		原因
	*/
	public DbException( String msg, Throwable ex ) {
		super( msg, ex );
	}

	/**
	* 指定された詳細メッセージ、SQL、原因を使用して、DbExceptionを構築します。
	* @param	msg		詳細メッセージ
	* @param	sql		SQL
	* @param	ex		原因
	*/
	public DbException( String msg, String sql, Throwable ex ) {
		super( msg, ex );
		this.sql = sql;
	}

	/**
	* この DbException オブジェクトのエラーメッセージ文字列を返します。
	* @return	エラーメッセージ文字列付きで作成された場合は、
	*			この DbException オブジェクトのエラーメッセージ文字列。
	*			エラーメッセージなしで作成された場合は null
	*/
	public String getMessage() {
		String	str;
		str = super.getMessage();
		if ( getCause() != null ) {
			if ( getCause() instanceof SQLException ) {
				SQLException se = (SQLException )getCause();
				str = str + ":" + se.getMessage() + ":" + se.getErrorCode() + ":";
			} else {
				str = str + ":" + getCause().toString() + ":";
			}
		}
		if ( sql != null ) {
			str = str + "sql[" + sql + "]";
		}
		return str;
	}

	/**
	* この DbException オブジェクトのベンダー固有の例外コードを取得します。
	* @return	ベンダーのエラーコード
	*/
	public int getErrorCode() {
		if ( getCause() != null ) {
			if ( getCause() instanceof SQLException ) {
				SQLException se = (SQLException )getCause();
				return se.getErrorCode();
			}
		}
		return -1;
	}

	public String getSql() {
		return sql;
	}

}
