package jp.co.sds_corp.util.diag;

import java.lang.management.ClassLoadingMXBean;
import java.lang.management.CompilationMXBean;
import java.lang.management.GarbageCollectorMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class JvmDiagnostics
{
	static final CmnLogger logger = CmnLogFactory.getLogger();

	public void displayClassLoader() {
		List<String> list = getClassLoader();
		for (String msg : list) {
			logger.info(this, msg);
		}
	}
	public List<String> getClassLoader() {
		ArrayList<String> ret = new ArrayList<String>();
		ClassLoadingMXBean clBean = ManagementFactory.getClassLoadingMXBean();
		ret.add("ClassLoader loaded=" + clBean.getLoadedClassCount()
				+ " total=" + clBean.getTotalLoadedClassCount()
				+ " unload=" + clBean.getUnloadedClassCount());
		return ret;
	}

	public void displayCompilation() {
		List<String> list = getCompilation();
		for (String msg : list) {
			logger.info(this, msg);
		}
	}
	public List<String> getCompilation() {
		ArrayList<String> ret = new ArrayList<String>();
		CompilationMXBean compBean = ManagementFactory.getCompilationMXBean();
		ret.add("JIT " + compBean.getName()
				+ " compile time=" + compBean.getTotalCompilationTime());
		return ret;
	}

	public void displayGarbageCollector() {
		List<String> list = getGarbageCollector();
		for (String msg : list) {
			logger.info(this, msg);
		}
	}
	public List<String> getGarbageCollector() {
		ArrayList<String> ret = new ArrayList<String>();
		List<GarbageCollectorMXBean> list = ManagementFactory.getGarbageCollectorMXBeans();
		for (GarbageCollectorMXBean gcBean : list) {
			ret.add(getGarbageCollector(gcBean));
		}
		return ret;
	}

	String getGarbageCollector(GarbageCollectorMXBean gcBean) {
		StringBuilder buf = new StringBuilder();
		buf.append("GC " + gcBean.getName()
			+ " count=" + gcBean.getCollectionCount()
			+ " time=" + gcBean.getCollectionTime()
			+ " pool=");
		String[] pools = gcBean.getMemoryPoolNames();
		for (String pool : pools) {
			buf.append(pool + ",");
		}
		return buf.toString();
	}

	public void displayRuntime() {
		List<String> ret = getRuntime();
		for (String msg : ret) {
			logger.info(this, msg);
		}
	}
	public List<String> getRuntime() {
		ArrayList<String> ret = new ArrayList<String>();
		RuntimeMXBean rtBean = ManagementFactory.getRuntimeMXBean();
		Date start = new Date(rtBean.getStartTime());
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.SSS");
		ret.add("Runtime " + rtBean.getName()
				+ " from " + sdf.format(start)
				+ " uptime=" + rtBean.getUptime());
		ret.add("bootClassPath=" + rtBean.getBootClassPath());
		List<String> args = rtBean.getInputArguments();
		for (String arg : args) {
			ret.add("arg=" + arg);
		}
		return ret;
	}

}
