package jp.co.sds_corp.util.diag;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryManagerMXBean;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.MemoryUsage;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class MemoryDiagnostics
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	DecimalFormat df = new DecimalFormat("#,##0");

	public void displayMemoryInfo() {
		List<String> ret = getMemoryInfo();
		for (String msg : ret) {
			logger.info(this, msg);
		}
	}
	public List<String> getMemoryInfo() {
		ArrayList<String> ret = new ArrayList<String>();
		Runtime runtime = Runtime.getRuntime();
		ret.add("=== memory info ===");
		ret.add("free=" + runtime.freeMemory()
				+ " total=" + runtime.totalMemory()
				+ " max=" + runtime.maxMemory());

		ret.addAll(getHeap());
//		ret.addAll(getMemoryManager());
		ret.addAll(getMemoryPool());
		return ret;
	}

	List<String> getHeap() {
		ArrayList<String> ret = new ArrayList<String>();
		MemoryMXBean memoryBean = ManagementFactory.getMemoryMXBean();
		MemoryUsage heap = memoryBean.getHeapMemoryUsage();
		ret.add("Heap " + getString(heap));
		MemoryUsage nonHeap = memoryBean.getHeapMemoryUsage();
		ret.add("NonHeap " + getString(nonHeap));
		return ret;
	}

	List<String> getMemoryManager() {
		ArrayList<String> ret = new ArrayList<String>();
		List<MemoryManagerMXBean> list = ManagementFactory.getMemoryManagerMXBeans();
		for (MemoryManagerMXBean manager : list) {
			ret.add("manager " + manager.getName());
		}
		return ret;
	}

	List<String> getMemoryPool() {
		ArrayList<String> ret = new ArrayList<String>();
		List<MemoryPoolMXBean> list = ManagementFactory.getMemoryPoolMXBeans();
		for (MemoryPoolMXBean pool : list) {
			ret.add("pool \"" + pool.getName() + "\""
					+ " type=" + pool.getType()
					+ " " + getString(pool.getUsage()));
		}
		return ret;
	}

	String getString(MemoryUsage usage) {
		if (usage == null) {
			return "";
		}
		String ret = "used="  + usage.getUsed()
			+ " : init=" + usage.getInit()
 			+ " committed="  + usage.getCommitted()
			+ " max="  + usage.getMax();
		return ret;
	}

}
