package	jp.co.sds_corp.util.math;


/**
３次方程式の解法(カルダノの公式) 
ax<SUP>3</SUP>+bx<SUP>2</SUP>+cx+d = 0 の解き方。
*/
public class Cardano
{
	/**
	* 3次元方程式をカルダノの公式を使用して解く。
	* @param	a		x<SUP>3</SUP>の係数
	* @param	b		x<SUP>2</SUP>の係数
	* @param	c		xの係数
	* @param	d		係数
	*/
	public static void solve(double a, double b, double c, double d) {
		b /= (3 * a);
		c /= a;
		d /= a;
		double p = b * b - c / 3;
		double q = (b * (c - 2 * b * b) - d) / 2;

		a = q * q - p * p * p;
		if (a == 0) {
			q = CubicRoot.doubleCuberoot(q);
			double x1 = 2 * q - b;
			double x2 = -q - b;
			System.out.println("x = " + x1 + ", " + x2 + " (重解) ");
		} else if (a > 0) {
			double a3;
			if (q > 0) {
				a3 = CubicRoot.doubleCuberoot(q + Math.sqrt(a));
			} else {
				a3 = CubicRoot.doubleCuberoot(q - Math.sqrt(a));
			}
			double b3 = p / a3;
			double x1 = a3 + b3 - b;
			double x2 = -0.5 * (a3 + b3) - b;
			double x3 = Math.abs(a3 - b3) * Math.sqrt(3.0) / 2;
			System.out.println("x = " + x1 + "; " + x2 + " +- " + x3 + " i");
		} else {
			a = Math.sqrt(p);
			double t = Math.acos(q / (p * a));
			a *= 2;
			double x1 = a * Math.cos(t                 / 3) - b;
			double x2 = a * Math.cos((t + 2 * Math.PI) / 3) - b;
			double x3 = a * Math.cos((t + 4 * Math.PI) / 3) - b;
			System.out.println("x = " + x1 + ", " + x2 + "," + x3);
		}
	}
}
