package	jp.co.sds_corp.util.math;

import java.io.IOException;
import java.io.StreamTokenizer;

/**
* 多変数多項式を解く。
*/
public class Polynomials
{
	public static final int N_LETTER = 4;
	public static class Node {
		int expo[];
		long real;
		long imag;
		Node next;

		private Node() {
			this(0, 0, null);
		}
		private Node(long real, long imag, Node next) {
			this.real = real;
			this.imag = imag;
			this.next = next;
			expo = new int[N_LETTER];
			for (int i=0; i<N_LETTER; i++) {
				expo[i] = 0;
			}
		}
	}

	public static int table[] = new int[26];
	public static Node value[] = new  Node[26];
	public static char letter[] = new char[N_LETTER];
	public static int cells;
	public static int maxCells;
	public static StreamTokenizer in;

	public static void error(String message) {
		System.out.println("\n" + message);
		System.exit(1);
	}

	public static boolean isOdd(int n) {
		return (n & 1) != 0;
	}
	public static int alphabetOrder(int c) {
		return Character.digit((char)c, 36) - 10;
	}

	public static Node newNode() {
		return newNode(0, 0, null);
	}
	public static Node newNode(long real, long imag, Node next) {
		Node p = new Node(real, imag, next);
		if (++cells > maxCells) {
			maxCells = cells;
		}
		return p;
	}

	public static void disposeNode(Node p) {
		cells--;
	}

	public static void dispose(Node p) {
		cells--;
		while (p.next != null) {
			p = p.next;
			cells--;
		}
	}

	public static Node constant(long re, long im) {
		Node p = newNode();
		if (re != 0 || im != 0) {
			p.next = newNode(re, im, null);
		}
		return p;
	}

	public static Node copy(Node p) {
		Node q = newNode(), r = q;
		while ((p = p.next) != null) {
			r = r.next = newNode(p.real, p.imag, null);
			for (int i=0; i<N_LETTER; i++) {
				r.expo[i] = p.expo[i];
			}
		}
		r.next = null;
		return q;
	}

	public static void changeSign(Node p) {
		while ((p = p.next) != null) {
			p.real = -p.real;
			p.imag = -p.imag;
		}
	}

	public static void differentiate(Node p) {
		if (alphabetOrder(in.ttype) < 0) {
			error("文字がありません");
		}

		int j = table[alphabetOrder(in.ttype)];

		if (j < 0) {
			error("微分できません");
		}

		Node p1 = p; p = p.next;
		while (p != null) {
			int e = p.expo[j];
			if (e != 0) {
				p.expo[j] = e - 1;
				p.real *= e;
				p.imag *= e;
				p1 = p;
				p = p.next;
			} else {
				p = p.next; disposeNode(p1.next); p1.next = p;
			}
		}
	}

	public static void complexConjugate(Node p) {
		while ((p = p.next) != null) {
			p.imag = -p.imag;
		}
	}

	public static void add(Node p, Node q) {
		int ep = 0, eq = 0;
		Node p1 = p; p = p.next;
		Node q1 = q; q = q.next;

		disposeNode(q1);
		while (q != null) {
			while (p != null) {
				for (int i=0; i<N_LETTER; i++) {
					ep = p.expo[i];
					eq = q.expo[i];
					if (ep != eq) {
						break;
					}
				}
				if (ep <= eq) {
					break;
				}
				p1 = p;
				p = p.next;
			}
			if (p == null || ep < eq) {
				p1.next = q;
				p1 = q;
				q = q.next;
				p1.next = p;
			} else {
				p.real += q.real;
				p.imag += q.imag;
				if (p.real != 0 || p.imag != 0) {
					p1 = p;
					p = p.next;
				} else {
					p = p.next;
					disposeNode(p1.next);
					p1.next = p;
				}
				q1 = q;
				q = q.next;
				disposeNode(q1);
			}
		}
	}

	public static Node multiply(Node x, Node y) {
		int ep = 0, eq = 0;
		Node q = null;
		Node r = newNode();
		r.next = null;

		while ((y = y.next) != null) {
			Node p1 = r;
			Node p = p1.next;
			Node z = x;
			while ((z = z.next) != null) {
				if (q == null) {
					q = newNode();
				}
				q.real = y.real * z.real - y.imag * z.imag;
				q.imag = y.real * z.imag + y.imag * z.real;
				for (int i=0; i<N_LETTER; i++) {
					q.expo[i] = y.expo[i] + z.expo[i];
				}

				while (p != null) {
					for (int i=0; i<N_LETTER; i++) {
						ep = p.expo[i];
						eq = q.expo[i];
						if (ep != eq) {
							break;
						}
					}
					if (ep <= eq) {
						break;
					}
					p1 = p;
					p = p.next;
				}

				if (p == null || ep < eq) {
					p1.next = q;
					p1 = q;
					p1.next = p;
					q = null;
				} else {
					p.real += q.real;
					p.imag += q.imag;
					if (p.real != 0 || p.imag != 0) {
						p1 = p;
						p = p.next;
					} else {
						p = p.next;
						disposeNode(p1.next);
						p1.next = p;
					}
				}
			}
		}
		if (q != null) {
			disposeNode(q);
		}
		return r;
	}

	public static Node power(Node x, int n) {
		if (n == 1) {
			return x;
		}

		Node p;
		if (n == 0) {
			p = constant(1, 0);
		} else {
			p = multiply(x, x);
			n -= 2;
			if (n > 0) {
				Node q = p;
				if (isOdd(n)) {
					p = multiply(q, x);
				} else {
					p = copy(q);
				}
				dispose(x);
				x = q;
				n /= 2;
				if (isOdd(n)) {
					q = multiply(p, x);
					dispose(p);
					p = q;
				}
				while ((n /= 2) != 0) {
					q = multiply(x, x);
					dispose(x);
					x = q;
					if (isOdd(n)) {
						q = multiply(p, x);
						dispose(p);
						p = q;
					}
				}
			}
		}
		dispose(x);
		return p;
	}

	public static void sincos(Node x) {
		int i;
		do {
			i = 0;
			Node p1 = x;
			Node p = p1.next;
			Node q = newNode();
			Node r = q;
			Node s = newNode();
			Node t = s;

			while (p != null) {
				if (p.expo[1] >= 2) {
					r = r.next = newNode();
					r.real = -p.real;
					r.imag = -p.imag;
					r.expo[0] = p.expo[0] + 2;
					r.expo[1] = p.expo[1] -= 2;
					for (i=2; i<N_LETTER; i++) {
						r.expo[i] = p.expo[i];
					}
					t = t.next = p;
					p1.next = p = p.next;
				} else {
					p1 = p;
					p = p.next;
				}
			}
			if (i != 0) {
				r.next = t.next = null;
				add(x, q);
				add(x, s);
			} else {
				disposeNode(q);
				disposeNode(s);
			}
		} while (i != 0);
	}

	public static Node variable() throws IOException {
		Node p;
		int i = alphabetOrder(in.ttype);
		int j = table[i];
		if (j >= 0) {
			p = constant(1, 0);
			int num = 1;
			if (in.nextToken() == '^') {
				if (in.nextToken() != StreamTokenizer.TT_NUMBER) {
					error("数がありません");
				}
				num = (int)in.nval;
				in.nextToken();
			}
			(p.next).expo[j] = (int)num;
		} else {
			if (value[i] == null) {
				error("未定義の文字です");
			}
			p = copy(value[i]);
			in.nextToken();
		}
		return p;
	}

	public static Node factor() throws IOException {
		Node p = null;

		if (in.ttype == '(') {
			in.nextToken();
			p = expression();
			if (in.ttype != ')') {
				error((char)in.ttype + "の前に')'がありません");
			}
			in.nextToken();
		} else if (in.ttype == StreamTokenizer.TT_NUMBER) {
			long num = (long)in.nval;
			if (Character.toUpperCase((char)in.nextToken()) != 'I') {
				p = constant(num, 0);
			} else {
				in.nextToken();
				p = constant(0, num);
			}
		} else if (alphabetOrder(in.ttype) >= 0) {
			p = variable();
		} else {
			error("因子の文法が間違っています");
		}

		for (;;) {
			switch (in.ttype) {
			case '^' :
				if (in.nextToken() != StreamTokenizer.TT_NUMBER) {
					error("数がありません");
				}
				p = power(p, (int)in.nval);
				in.nextToken();
				break;
			case ':' :
				in.nextToken();
				differentiate(p);
				in.nextToken();
				break;
			case '!' :
				in.nextToken();
				sincos(p);
				break;
			case '\'':
				in.nextToken();
				complexConjugate(p);
				break;
			default:
				return p;
			}
		}
	}

	public static Node term() throws IOException {
		Node p = factor();
		while (in.ttype == '*') {
			in.nextToken();
			Node q = p;
			Node r = factor();
			p = multiply(q, r);
			dispose(q);
			dispose(r);
		}
		return p;
	}

	public static Node expression() throws IOException {
		Node p, q;
		if (in.ttype == '-') {
			in.nextToken();
			p = term();
			changeSign(p);
		} else {
			if (in.ttype == '+') {
				in.nextToken();
			}
			p = term();
		}

		for (;;) {
			switch (in.ttype) {
			case '+':
				in.nextToken();
				q = term();
				break;
			case '-':
				in.nextToken();
				q = term();
				changeSign(q);
				break;
			default:
				return p;
			}
			add(p, q);
		}
	}

	public static void initialize() {
		for (int i=0; i<table.length; i++) {
			table[i] = -1;
		}
	}

	public static void declare() throws IOException {
		for (int i=0; i<table.length; i++) {
			table[i] = -1;
			if (value[i] != null) {
				dispose(value[i]);
				value[i] = null;
			}
		}

		int j = 0;
		in.nextToken();
		while (alphabetOrder(in.ttype) >= 0) {
			int i = alphabetOrder(in.ttype);
			if (table[i] < 0) {
				if (j >= N_LETTER) {
					error("文字が多すぎます");
				}
				table[i] = j;
				letter[j] = (char)in.ttype;
				j++;
			}
			if (in.nextToken() == ',') {
				in.nextToken();
			}
		}
	}

	public static void assign() throws IOException {
		int i = alphabetOrder(in.ttype);
		if (table[i] >= 0) {
			error("左辺が間違っています");
		}
		if (in.nextToken() != '=') {
			error("'='がありません");
		}
		in.nextToken();
		Node p = expression();
		if (value[i] != null) {
			dispose(value[i]);
		}
		value[i] = p;
	}

	public static void print() throws IOException {
		boolean first = true;

		in.nextToken();
		Node p = expression(), q = p;
		System.out.println();
		while ((p = p.next) != null) {
			boolean one = false;
			long re = p.real, im = p.imag;
			if (im == 0) {
				if (re >= 0) {
					if (!first) {
						System.out.print(" + ");
					}
				} else {
					System.out.print(" - "); re = -re;
				}
				if (re == 1) {
					one = true;
				} else {
					System.out.print(re);
				}
			} else if (re == 0) {
				if (im >= 0) {
					if (!first) {
						System.out.print(" + ");
					} else {
						System.out.print(" - "); im = -im;
					}
				}
				System.out.print(im + "i");
			} else {
				if (!first) {
					System.out.print(" + ");
				}
				System.out.print("(" + re + (im < 0 ? "" : "+") + im + "i)");
			}
			first = false;
			for (int i=0; i<N_LETTER; i++) {
				int e = p.expo[i];
				if (e != 0) {
					if (!one) {
						System.out.print(" * ");
					}
					one = false;
					System.out.print(letter[i]);
					if (e != 1) {
						System.out.print("^" + e);
					}
				}
			}
			if (one) {
				System.out.print("1");
			}
		}
		if (first) {
			System.out.print("0");
		}
		System.out.println();
		dispose(q);
	}
}
