package jp.co.sds_corp.util.math;

/**
* 関数f(x)の解を逐次法により求めます。
*/
public class Serial {
	/**
	* 関数f(x)
	*/
	public interface Function {
		public double of(double x) throws UtilMathException;
	}
    public static final int PLUS_DIRECTION  = 0;
    public static final int MINUS_DIRECTION = 1;

	/*
	* 逐次探索
	* @param f 計算関数
	* @param s 開始値
	* @param e 終了値
	* @param t ステップ数
	* @param r 切捨て情報
	* @param eps 収束値
	* @param d 探索方向
	* @return  収束値
	*/
	public static double solve(Function f, double s, double e,
			double t, double floor, double eps, int d)
		throws UtilMathException
	{
		double bk_y=0;
		double bk_x=0;
		double min_x=0;
		double min_y=0;
		double y=0;
		double x=0;
		double r=0;
		x = s;
		for (int i=0; ; i++) {
			y = f.of(x);
			if (i == 0) {
				bk_y = y;
				bk_x = x;
				if (y > 0) {
					min_y = y;
					min_x = x;
				} else {
					min_y = 1.0;
					min_x = 1000000.0;
				}
			}

			if (y == 0) { r = x; break; }
			if ((y >= 0 && bk_y <= 0) || (y <= 0 && bk_y >= 0)) {
				if (y >=0) {
					r = x;
				} else {
					r = bk_x;
					y = bk_y;
				}
				break;
			} else {
				bk_y = y;
				bk_x = x;
				if (y > 0) {
					if (Math.abs(min_y) > Math.abs(y)) {
						min_y = y;
						min_x = x;
					}
				}
			}
			x += (t);
			x = Math.floor(x * floor + 0.00001) / floor;
			if (fin(x, e, d)) {	
				y = min_y;
				r = min_x;
				break;
			}
		}
		if (y > eps) {
			throw new UtilMathException(UtilMathException.SERIAL_EXCEPTION,
								"値が収束しませんでした。", new Exception());
		} else {
			if (y < 0) {
				throw new UtilMathException(UtilMathException.SERIAL_EXCEPTION,
									"値が収束しませんでした。", new Exception());
			}
		}
		return r;
	}

	private static boolean fin(double x, double e, int d) {
		if (d == PLUS_DIRECTION) {
			return (x >= e) ? true : false;
		} else {
			return (x <= e) ? true : false;
		}
	}
}
