package	jp.co.sds_corp.util.script;

import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;

/**
{@link IScriptParser}を生成するFactoryクラスです。
以下の形式のXMLファイルの、languageによって{@link IScriptParser}を生成します。
<pre>
&lt;Script
	language="JavaScript"&gt;
&lt;/Script&gt;
</pre>
languageの代わりにclass属性が指定された場合は、指定されたオブジェクトを生成します。
<pre>
&lt;Script
	class="<EM>foo.bar.class</EM>"&gt;
&lt;/Script&gt;
</pre>
*/
public class ScriptParserFactory
{
	/**
	* 設定情報により、{@link IScriptParser}を生成します。
	* @param	config	設定情報
	* @return	生成された{@link IScriptParser}
	*/
	public static IScriptParser createParser( IConfiguration config )
		throws Exception
	{
		return createParser( config, null );
	}

	/**
	* 設定情報により、{@link IScriptParser}を生成します。
	* @param	config	設定情報
	* @param	prop	properties
	* @return	生成された{@link IScriptParser}
	*/
	public static IScriptParser createParser( IConfiguration config, CmnProperty prop )
		throws Exception
	{
		IScriptParser ret = null;
		String lang = config.getAttributes().get( "language", null );
		String cls = config.getAttributes().get( "class", null );
		if ( lang != null ) {
			if ( lang.equals( "JavaScript" )) {
				ret = new JsParser();
			} else {
				throw( new IllegalArgumentException( "illegal language.[" + lang + "]" ));
			}
		} else if ( cls != null ) {
			ret = (IScriptParser )Class.forName( cls ).newInstance();
		}
		if ( ret == null ) {
			throw( new IllegalArgumentException( "language/class not defined." ));
		}
		ret.setProperty( prop );
		ret.init( config );
		return ret;
	}
}
