package jp.co.sds_corp.util.syslog;

import jp.co.sds_corp.util.common.NoSuchCodeException;

/**
* syslogの分類を示すクラスです。
* @author	小見山　聡
*/
public enum Facility
{
	/** kernel messages */
	KERN( 0 ),
	/** random user-level messages */
	USER( 1 ),
	/** mail system */
	MAIL( 2 ),
	/** system daemons */
	DAEMON( 3 ),
	/** security/authorization messages */
	AUTH( 4 ),
	/** messages generated internally by syslogd */
	SYSLOG( 5 ),
	/** line printer subsystem */
	LPR( 6 ),
	/** netnews subsystem */
	NEWS( 7 ),
	/** uucp subsystem */
	UUCP( 8 ),
	/** cron/at subsystem */
	CRON( 9 ),
	/** security/authorization messages (private) */
	AUTHPRIV( 10 ),
	/** ftp daemon */
	FTP( 11 ),

    /* other codes through 15 reserved for system use */
	/** reserved for local use */
	LOCAL0( 16 ),
	/** reserved for local use */
	LOCAL1( 17 ),
	/** reserved for local use */
	LOCAL2( 18 ),
	/** reserved for local use */
	LOCAL3( 19 ),
	/** reserved for local use */
	LOCAL4( 20 ),
	/** reserved for local use */
	LOCAL5( 21 ),
	/** reserved for local use */
	LOCAL6( 22 ),
	/** reserved for local use */
	LOCAL7( 23 );

	/** maximum number of facilities */
	public static final int	LOG_NFACILITIES = 24;
	/** mask to extract facility part */
	public static final int	LOG_FACMASK = 0x03f8;

	int		value;

	/**
	* コンストラクタ
	*/
	Facility( int var ) {
		this.value = var;
	}

	/**
	* 出力元の一覧を返します。
	* @return	出力元の一覧
	*/
	public static String[] listMessages() {
		String[] ret = new String[ Facility.values().length ];
		for ( int i = 0; i < Facility.values().length; i++ ) {
			ret[i] = Facility.values()[ i ].toString();
		}
		return ret;
	}

	/**
	* 出力元の情報を返します。
	* @return	出力元
	*/
	public int getValue() {
		return value;
	}

	/**
	* 数値をPriorityに変換します。
	* @param	var		数値
	* @return	変換後の値。変換できなければnull
	*/
	public static Facility parse( int var ) throws NoSuchCodeException {
		for ( Facility fac : Facility.values() ) {
			if ( fac.getValue() == var ) {
				return fac;
			}
		}
		throw( new NoSuchCodeException( "unknown facility [" + var + "]" ));
	}

	/**
	* 文字列をPriorityに変換します。
	* @param	code	文字列
	* @return	変換後の値。変換できなければnull
	*/
	public static Facility parse( String code ) throws NoSuchCodeException {
		if ( code == null ) {
			return null;
		}
		String upper = code.toUpperCase();
		for ( Facility fac : Facility.values() ) {
			if ( fac.toString().equals( upper )) {
				return fac;
			}
		}
		throw( new NoSuchCodeException( "unknown facility [" + code + "]" ));
	}

}
