package jp.co.sds_corp.util.syslog;

import jp.co.sds_corp.util.common.NoSuchCodeException;

/**
* syslogの重要度を示すクラスです。
* @author	小見山　聡
*/
public enum Priority
{
	/* Priorities (these are ordered) */
	/** system is unusable */
	EMERG( 0 ),
	/** action must be taken immediately */
	ALERT( 1 ),
	/** critical conditions */
	CRIT( 2 ),
	/** error conditions */
	ERR( 3 ),
	/** warning conditions */
	WARNING( 4 ),
	/** normal but signification condition */
	NOTICE( 5 ),
	/** informational */
	INFO( 6 ),
	/** debug-level messages */
	DEBUG( 7 );

	/* mask to extract priority part (internal) */
	public static final int	LOG_PRIMASK = 0x0007;

	int value;

	/**
	* コンストラクタ
	*/
	Priority( int var ) {
		this.value = var;
	}

	/**
	* 重要度の一覧を返します。
	* @return	重要度の一覧
	*/
	public static String[] listMessages() {
		String[] ret = new String[ Priority.values().length ];
		for ( int i = 0; i < Priority.values().length; i++ ) {
			ret[i] = Priority.values()[ i ].toString();
		}
		return ret;
	}

	/**
	* 重要度の情報を返します。
	* @return	重要度
	*/
	public int getValue() {
		return this.value;
	}

	/**
	* 数値をPriorityに変換します。
	* @param	var		数値
	* @return	変換後の値。変換できなければnull
	*/
	public static Priority parse( int var ) throws NoSuchCodeException {
		for ( Priority pri : Priority.values() ) {
			if ( pri.getValue() == var ) {
				return pri;
			}
		}
		throw( new NoSuchCodeException( "unknown priority [" + var + "]" ));
	}

	/**
	* 文字列をPriorityに変換します。
	* @param	code	文字列
	* @return	変換後の値。変換できなければnull
	*/
	public static Priority parse( String code ) throws NoSuchCodeException {
		if ( code == null ) {
			return null;
		}
		String upper = code.toUpperCase();
		for ( Priority pri : Priority.values() ) {
			if ( pri.toString().equals( upper )) {
				return pri;
			}
		}
		throw( new NoSuchCodeException( "unknown priority [" + code + "]" ));
	}

}
