package jp.co.sds_corp.util.crypt;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class CipherFactoryTest extends BasicTestCase
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	IConfiguration doc;
	IConfiguration config;
	static final String plain = "abc123";

	public void setUp() throws Exception {
		super.setUp();
		doc = DOMConfiguration.findResource("cipher.xml");
	}

	public void testCipher1() throws Exception {
		logger.debug(this, "===== testCipher1");
		config = doc.getChild("cipher", "name", "private1");
		ICipher cipher = CipherFactory.createCipher(config);
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		config = doc.getChild("cipher", "name", "public1");
		ICipher decipher = CipherFactory.createCipher(config);
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);

		assertEquals("decrypted", plain, sdec);
	}

	public void testCipher2() throws Exception {
		logger.debug(this, "===== testCipher2");
		config = doc.getChild("cipher", "name", "private2");
		ICipher cipher = CipherFactory.createCipher(config);
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		config = doc.getChild("cipher", "name", "public1");
		ICipher decipher = CipherFactory.createCipher(config);
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);

		assertEquals("decrypted", plain, sdec);
	}

	public void testKeystore1() throws Exception {
		logger.debug(this, "===== testKeystore1");
		config = doc.getChild("cipher", "name", "keystore1");
		ICipher cipher = CipherFactory.createCipher(config);
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		config = doc.getChild("cipher", "name", "keystore2");
		ICipher decipher = CipherFactory.createCipher(config);
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);

		assertEquals("decrypted", plain, sdec);
	}

	public void testPKCS12_1() throws Exception {
		logger.debug(this, "===== testPKCS12_1");
		config = doc.getChild("cipher", "name", "pkcs12_1");
		ICipher cipher = CipherFactory.createCipher(config);
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		config = doc.getChild("cipher", "name", "pkcs12_2");
		ICipher decipher = CipherFactory.createCipher(config);
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);

		assertEquals("decrypted", plain, sdec);
	}

/*
	public void testMain1() throws Exception {
		logger.debug("===== testMain1");
		InputStream in = new FileInputStream("target/test-classes/cipher.xml");
		CipherFactory.setInputStream(in);

		String[] args = {"cipher.xml", "private1", "encrypt"};
		CipherFactory.main(args);
	}
*/

}
