package jp.co.sds_corp.util.crypt;

import java.io.File;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class StoreCipherTest extends BasicTestCase
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	static final String plain = "abc123";

	public void setUp() throws Exception {
		super.setUp();
	}

	public void testKeystore1() throws Exception {
		logger.debug(this, "===== testKeystore1");
		StoreKeyCipher cipher = new StoreKeyCipher(new File("target/test-classes/keystore"));
		cipher.setStorePassword("xsw23edc");
		cipher.setAlias("sample");
		cipher.setPassword("zaq12wsx");
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		StoreCertCipher decipher = new StoreCertCipher(new File("target/test-classes/keystore"));
		decipher.setStorePassword("xsw23edc");
		decipher.setAlias("sample");
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);
		assertEquals("decrypted", plain, sdec);
	}

	public void testKeystore2() throws Exception {
		logger.debug(this, "===== testKeystore2");
		StoreCertCipher cipher = new StoreCertCipher(new File("target/test-classes/keystore"));
		cipher.setStorePassword("xsw23edc");
		cipher.setAlias("sample");
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		StoreKeyCipher decipher = new StoreKeyCipher(new File("target/test-classes/keystore"));
		decipher.setStorePassword("xsw23edc");
		decipher.setAlias("sample");
		decipher.setPassword("zaq12wsx");
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);
		assertEquals("decrypted", plain, sdec);
	}

	public void testPKCS12_1() throws Exception {
		logger.debug(this, "===== testPKCS12_1");
		StoreKeyCipher cipher = new StoreKeyCipher(new File("target/test-classes/cert.p12"));
		cipher.setStoreType("PKCS12");
		cipher.setStorePassword("zaq12wsx");
		cipher.setAlias("sample");
		cipher.setPassword("zaq12wsx");
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug(this, "encrypted=" + new Base64Encoder().encode(encrypted));

		StoreCertCipher decipher = new StoreCertCipher(new File("target/test-classes/cert.p12"));
		decipher.setStoreType("PKCS12");
		decipher.setStorePassword("zaq12wsx");
		decipher.setAlias("sample");
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug(this, "decrypted=" + sdec);
		assertEquals("decrypted", plain, sdec);
	}

}
