package jp.co.sds_corp.util.log;

import java.text.DecimalFormat;
import java.util.logging.Level;

import jp.co.sds_corp.test.BasicTestCase;

public class CmnLoggerTest extends BasicTestCase
{
	static final ConsoleLogger console = CmnLogFactory.getConsoleLogger();
	static final CmnLogger logger = CmnLogFactory.getLogger();

	public void setUp() throws Exception {
		super.setUp();

		console.setLevel( Level.ALL );
	}

    /**
     */
    public void testDebug1() throws Exception {
		console.log( this, "main thread=" + Thread.currentThread().getId() );
		logger.debug( this, "debug msg" );
    }

    /**
     */
    public void testDebug2() throws Exception {
		Exception ex = new Exception( "test exception" );
		logger.debug( this, "debug msg", ex );
    }

    /**
     */
    public void testDebug3() throws Exception {
    	CmnLogger logger = CmnLogFactory.getLogger();
    	logger.debug( this, "debug test msg" );
    }

    /**
     */
    public void testDebug4() throws Exception {
    	CmnLogger logger = CmnLogFactory.getLogger();
		logger.debug( this.getClass(), "debug test msg" );
    }

    /**
     */
    public void testInfo1() throws Exception {
		logger.info( this, "info msg" );
    }

    /**
     */
    public void testInfo2() throws Exception {
		Exception ex = new Exception( "test exception" );
		logger.info( this, "info msg", ex );
    }

    /**
     */
    public void testWarn1() throws Exception {
		logger.warn( this, "warn msg" );
    }

    /**
     */
    public void testWarn2() throws Exception {
		Exception ex = new Exception( "test exception" );
		logger.warn( this, "warn msg", ex );
    }

    /**
     */
    public void testError1() throws Exception {
		logger.error( this, "error msg" );
    }

    /**
     */
    public void testError2() throws Exception {
		Exception ex1 = new Exception( "test exception1" );
		Exception ex2 = new Exception( "test exception2", ex1 );
		logger.error( this, "error msg", ex2 );
    }

    /**
     */
    public void testSql() throws Exception {
		logger.sql( this, "select *** from table" );
		logger.sql( this, "select *** from table2" );
    }

    /**
     */
    public void testTest2() throws Exception {
		logger.info( "jp.co.sds_corp.TEST." + getClass().getName(), "[INCLUDE] test" );
    }

    /**
     */
    public void testTest3() throws Exception {
		logger.info( "jp.co.sds_corp.TEST." + getClass().getName(), "[not INCLUDE]" );
    }

    /**
     */
    public void testTest4() throws Exception {
		logger.info( "jp.co.sds_corp.TEST." + getClass().getName(), "[INCLUDE] and [EXCLUDE]" );
    }

    /**
     */
    public void testTest5() throws Exception {
		logger.debug( "jp.co.sds_corp.TEST." + getClass().getName(), "[INCLUDE] but DEBUG" );
    }

    /**
     */
    public void testLineWrap() throws Exception {
		StringBuilder logMsg = new StringBuilder();
		DecimalFormat df = new DecimalFormat( "000000000" );
		for ( int i = 0; i < 210; i++ ) {
			logMsg.append( df.format( i ) + "," );
		}
		logger.debug( this, logMsg.toString() );
    }

    /**
     */
    public void testNull1() throws Exception {
		logger.trace( this, "test", (Throwable )null );
		logger.debug( this, "test", (Throwable )null );
		logger.detail( this, "test", (Throwable )null );
		logger.info( this, "test", (Throwable )null );
		logger.warn( this, "test", (Throwable )null );
		logger.error( this, "test", (Throwable )null );
    }

    /**
     */
    public void testNull2() throws Exception {
		logger.exception( this, null );
    }

}
